/**
 *  MainRenderer
 *
 *  Copyright (C) 2005  Yves Radunz, Volker Grabsch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

import java.text.*;
import javax.media.opengl.*;
import javax.media.opengl.glu.*;
import com.sun.opengl.utils.GLUT;

public class MainRenderer extends SimpleRenderer
{
    double radius_szene = 6.37e6*2;

    double radius_erde = 6.37e6;
    double masse_erde = 5.97e24;
    double radius_satellit = 2e5;

    double hoehe_satellit = 12e6;
    double alarm = radius_satellit*10;

    double speed = 10;

    private boolean solid;
    private double time;
    private SatNet satnet;

    private int[] status_sizes;
    private String[] status_titles;

    public void init()
    {
	solid = true;
	time = 0;
	satnet = new SatNet(hoehe_satellit, alarm);
	for (int i=1; i<=5; i++)
	    {
		satnet.newsat();
	    }

	title = "Satelliten-Netz";

	status_sizes = new int[]{6, 9, 10, 10, 16};

	status_titles = new String[]{
	    "Anzahl",
	    "Netzlänge",
	    "Min. Länge",
	    "Max. Länge",
	    "Durchschn. Länge",
	};

	initText = new String[]{
	    "",
	    "                        =================",
	    "                         Satelliten-Netz",
	    "                        =================",
	    "",
	    "  (C) 2005  Yves Radunz, Volker Grabsch",
	    "",
	    ".--------------------------- Tastenkürzel: ------------------<>-------.",
	    "|  F2, S   =  anhalten / weiter                                       |",
	    "|  F3, D   =  Erde durchsichtig  [an/aus]                             |",
	    "|  F4, *   =  neuen Satelliten einfügen (zufällig)                    |",
	    "|  F5, +   =  neuen Satelliten einfügen (minimale Kollisions-Gefahr)  |",
	    "|  F6, -   =  letzten Satelliten entfernen                            |",
	    "|  Esc, Q  =  beenden                                                 |",
	    "`-------------------------------------------------------- [<<<>>>] ---'",
	    "",
	    ".--------------------------- Mausaktionen: ------------------<>-------.",
	    "|  klicken,ziehen  =  Szene drehen                                    |",
	    "|  Doppelklick     =  Erde durchsichtig  [an/aus]                     |",
	    "`-------------------------------------------------------- [<<<>>>] ---'",
	    "",
	    "   Einheit: 10^6 Meter",
	    createStatusHeader(status_titles, status_sizes, '_'),
	    createStatusLine(status_titles, status_sizes),
	    createStatusHeader(status_titles, status_sizes, '-'),
	};

	doneText = new String[]{
	    createStatusHeader(status_titles, status_sizes, '^'),
	    "",
	    ".------------------------------------------------------------<>-------.",
	    "|  Auf Wiedersehen!                                                   |",
	    "`-------------------------------------------------------- [<<<>>>] ---'",
	};
    }

    /**
     * Render scene
     */
    public void glRender(GL gl)
    {
	final NumberFormat format = new DecimalFormat("##0.00");

	GLU glu = new GLU();
	GLUT glut = new GLUT();

	time += speed;


	boolean[] kollidiert = satnet.kollisionen(time);
	Graph graph = satnet.createnet(time);
	double graphlen = graph.getNetLength();

	boolean kollisionsgefahr = false;
	for (int i=0; i<kollidiert.length; i++)
	    kollisionsgefahr |= kollidiert[i];


	// Status-Zeile
	String line = "";
	if (kollisionsgefahr)
	    {
		line += "  <<< Kollisionsgefahr! >>>";
	    }
	if (graphlen <= 0)
	    {
		line += "  <<< Kein Netz! >>>";
	    }
	if (line.length() > 0)
	    {
		line = createStatusLine(new String[]{
					    String.valueOf(satnet.size()),
					},
					status_sizes)
		    + line;
	    }
	else
	    {
		line = createStatusLine(new String[]{
					    String.valueOf(satnet.size()),
					    format.format(graphlen/1e6),
					    format.format(satnet.minlength()/1e6),
					    format.format(satnet.maxlength()/1e6),
					    format.format(satnet.avrlength()/1e6),
					},
					status_sizes);
	    }
	showLine(line);
	

 	// Globale Skalierung
	double scale_factor = 1.0/radius_szene;
	gl.glScaled(scale_factor, scale_factor, scale_factor);

	// Globales Licht
	float light_intensity = (float)(1.0/radius_szene);
	float[] light_color = new float[]{light_intensity, light_intensity, light_intensity, 1};
	gl.glLightfv(GL.GL_LIGHT0, GL.GL_POSITION, new float[]{2f, 5f, -5f, 0f}, 0);
	gl.glLightfv(GL.GL_LIGHT0, GL.GL_AMBIENT, light_color, 0);
	gl.glLightfv(GL.GL_LIGHT0, GL.GL_DIFFUSE, light_color, 0);
	gl.glLightfv(GL.GL_LIGHT0, GL.GL_SPECULAR, light_color, 0);

	// Erde
	gl.glPushMatrix();
	gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, new float[]{0.5f, 0.5f, 1.0f}, 0);
	gl.glRotated(90.0, 1, 0, 0);

	if (solid)
	    {
		// solide Erde
		glut.glutSolidSphere(radius_erde, 40, 40);
	    }
	else
	    {
		// durchsichtige Erde
		glut.glutWireSphere(radius_erde, 60, 60);
	    }

	// Erdachse
	gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, new float[]{0.7f, 1.0f, 0.7f}, 0);
	gl.glBegin(GL.GL_LINES);
	gl.glVertex3d(0, 0, 1.4*radius_erde);
	gl.glVertex3d(0, 0, -1.4*radius_erde);
	gl.glEnd();

	gl.glPopMatrix();

	// Satelliten
	for (int i=0, len=satnet.size(); i<len; i++)
	    {
		gl.glPushMatrix();
		if (kollidiert[i])
		    {
			// Kollisions-Farbe
			gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, new float[]{1.0f, 0.0f, 0.0f}, 0);
		    }
		else
		    {
			// normale Farbe
			gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, new float[]{1.0f, 0.5f, 0.5f}, 0);
		    }
		satnet.sat(i).glTranslate(gl, time);
		glut.glutSolidSphere((float)radius_satellit, 5, 5);
		gl.glPopMatrix();
	    }

	// Verbindungsnetz
	gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, new float[]{1.0f, 1.0f, 1.0f}, 0);
	gl.glBegin(GL.GL_LINES);
	for (int i=0, len=graph.size(); i<len; i++)
	    {
		int[] edge = graph.edge(i);
		satnet.sat(edge[0]).glVertex(gl, time);
		satnet.sat(edge[1]).glVertex(gl, time);
	    }
	gl.glEnd();
    }

    /**
     * React on double click
     */
    public void mouseDoubleClicked()
    {
	solid = !solid;
    }

    /**
     * React on key
     */
    public void keyTyped(char key)
    {
	switch (key)
	    {
	    case 'Q':
		quit();
		break;

	    case 'S':
		toggle();
		break;

	    case 'D':
		solid = !solid;
		break;

	    case '*':
		satnet.newsatrnd();
		break;

	    case '+':
		satnet.newsat();
		break;

	    case '-':
		satnet.removesat();
		break;
	    }
    }

    /**
     * React on function (F1, ...) key
     */
    public void keyFxTyped(int fkey)
    {
	switch (fkey)
	    {
	    case 2:
		toggle();
		break;

	    case 3:
		solid = !solid;
		break;

	    case 4:
		satnet.newsatrnd();
		break;

	    case 5:
		satnet.newsat();
		break;

	    case 6:
		satnet.removesat();
		break;
	    }
    }
}
